#include "BeNESJoypad.h"
#include <interface/InterfaceDefs.h>

inline bool isKeyDown(key_info& info, unsigned char key)
{
	return (info.key_states[key>>3] & (1<<(7-(key&7))));
}

#define RETURN 0x47
#define SPACE 0x5e
#define UP 0x57
#define DOWN 0x62
#define LEFT 0x61
#define RIGHT 0x63
#define LSHIFT 0x4b
#define Z 0x4c
#define X 0x4d
BeNESJoypad::BeNESJoypad(): NesJoypad()
{
	mThread = spawn_thread(BeNESJoypad::Polling, 
		"BeNESJoy", B_NORMAL_PRIORITY, this);
	resume_thread(mThread);
	mRunning = true;
}

BeNESJoypad::~BeNESJoypad()
{
	if (mThread) {
		status_t result;
		mRunning = false;
		wait_for_thread(mThread, &result);
		kill_thread(mThread);
	}
}

int32 BeNESJoypad::Polling(void* arg)
{
	const int pollingRate = 1000000 / 60;
	BeNESJoypad* joy = reinterpret_cast<BeNESJoypad*>(arg);
	key_info info; 
	while (joy->mRunning)
	{
		snooze(pollingRate);
		get_key_info(&info);
		
		if (isKeyDown(info, Z)) joy->pushA();
		else joy->releaseA();
		
		if (isKeyDown(info, X) || isKeyDown(info, LSHIFT)) joy->pushB();
		else joy->releaseB();

		if (isKeyDown(info, RETURN)) joy->pushStart();
		else joy->releaseStart();

		if (isKeyDown(info, SPACE)) joy->pushSelect();
		else joy->releaseSelect();

		if (isKeyDown(info, UP)) joy->pushUp();
		else joy->releaseUp();

		if (isKeyDown(info, DOWN)) joy->pushDown();
		else joy->releaseDown();

		if (isKeyDown(info, LEFT)) joy->pushLeft();
		else joy->releaseLeft();

		if (isKeyDown(info, RIGHT)) joy->pushRight();
		else joy->releaseRight();
	}
	return 0;
}
