/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <app/Application.h>
#include <app/Message.h>
#include <interface/Screen.h>
#include <string>
#include "BeNESPalette.h"
#include "BeNESWin.h"
#include "BeNESMenu.h"

#define RETURN 0x47
#define SPACE 0x5e
#define UP 0x57
#define DOWN 0x62
#define LEFT 0x61
#define RIGHT 0x63
#define Z 0x4c
#define X 0x4d

//--------------------------------------------------------------------------
BeNESWin::BeNESWin(): 
	BWindow(BRect(X, Y, X+W, Y+H), 
		"BeNES", 
		B_TITLED_WINDOW, 
		B_NOT_RESIZABLE),
	mOffscreen(NULL),
	mView(NULL),
	mMenubar(NULL),
	mFrameSnooze(0)
{
	Lock();
	mMenubar	= BeNESMenu::Build();
	AddChild(mMenubar);
	BRect barFrame(mMenubar->Frame());
	
	ResizeTo(W, H+barFrame.Height());

	mView = new BView(BRect(0, 0, W, H), NULL, B_FOLLOW_ALL, 0);
	AddChild(mView);
	mView->MoveBy(0, barFrame.Height()+1);
	mView->SetViewColor(0, 0, 0);
	
	mOffscreen = new BBitmap(BRect(0, 0, W-1, H-1), B_COLOR_8_BIT);
	mBits = static_cast<uchar*>(mOffscreen->Bits());
	mBytesPerRow = mOffscreen->BytesPerRow();
	BScreen* screen = new BScreen(this);
	memset(mBits, screen->IndexForColor(0, 0, 0, 255), W*H);
	for (int i=0; i<64; i++) {
		IndexPalette[i] = screen->IndexForColor(BeNESPalette[i]);
	}
	delete screen;
	Unlock();
}

//--------------------------------------------------------------------------
BMenu* BeNESWin::getSkipFramesMenu()
{
	BMenu* menu = mMenubar->SubmenuAt(1)->SubmenuAt(0);
	return menu;
}

//--------------------------------------------------------------------------
BMenu* BeNESWin::getFramesParSecMenu()
{
	BMenu* menu = mMenubar->SubmenuAt(1)->SubmenuAt(1);
	return menu;
}

//--------------------------------------------------------------------------
BMenu* BeNESWin::getSoundMenu()
{
	BMenu* menu = mMenubar->SubmenuAt(1)->SubmenuAt(2);
	return menu;
}

//--------------------------------------------------------------------------
void BeNESWin::setFPS(int fps)
{
	if (fps) {
		mFrameSnooze = 1000000 / fps;
	}
	else {
		mFrameSnooze = 0;
	}
}

//--------------------------------------------------------------------------
bool BeNESWin::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED, be_app);
	return true;
}

//--------------------------------------------------------------------------
void BeNESWin::updateScreen(const NesVirtualScreen* screen, bool draw)
{
	static bigtime_t prevTime = 0;
	if (draw) {
		static const int Y_MIN = NesDisplay::VisibleTop;
		static const int Y_MAX = NesDisplay::VisibleBottom;

		for (int y=Y_MIN; y<=Y_MAX; y++) {
			for (int x=8; x<256-8; x++) {
				*(mBits+(y*mBytesPerRow)+x) = 
					IndexPalette[screen->bits[y][x]];
			}
		}
		Lock();
		mView->DrawBitmap(mOffscreen);
		Unlock();
	}
	bigtime_t snooze_time = mFrameSnooze - (system_time()-prevTime);
	if (snooze_time > 0) {
		snooze(snooze_time);
	}
	prevTime = system_time();
}
