/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
// NES用 カスタム6502 CPUユニット
//
#ifndef NESCPU_H_DEFINED
#define NESCPU_H_DEFINED

extern "C" {
#include "M6502.h"
}
#include "NesPPURegisters.h"
#include "NesCPUInterface.h"

class NesIO;

class NesCPU: public NesCPUInterface {
	M6502 m6502;
	uchar mRam[0x10000];
	NesIO* mIO;
	bool mIsRunning;
	bool mFrameIrqEnabled;
	int mFrameClock;
	int mTicks;
public:
	NesCPU(NesIO* io): 
		mIO(io),
		mIsRunning(false),
		mFrameIrqEnabled(false),
		mFrameClock(0) { ; } 
	
	void reset();
	void run();
	void stop() { mIsRunning = false; }
public:
	virtual void memmove(size_t address, uchar* mem, size_t size) {
		memcpy(&(mRam[address]), mem, size);
	}
	virtual uchar* getSAVERAM() {
		return &(mRam[0x6000]);
	}
private:
	void clearRam();
	void setReadFunction();
	void setWriteFunction();
	
	// CPU INT
	void intNMI() { Int6502(&m6502, INT_NMI); }
	void intIRQ() { Int6502(&m6502, INT_IRQ); }

	// RAM IO
	static uchar ReadRam(void* user, int address);
	static void WriteRam(void* user, int address, uchar value);

	// PPU REGISTER IO
	static uchar ReadPPURegisters(void* user, int address);
	static void WritePPURegisters(void* user, int address, uchar value);
	
	// APU & JOY, SPRITE DMA
	static uchar Read0x4000(void* user, int address);
	static void Write0x4000(void* user, int address, uchar value);
	static void WriteAPURegisters(NesCPU* cpu, int address, uchar value);
	
	// Normal RAM Access
	static uchar ReadNormal(void* user, int address);
	static void WriteNormal(void* user, int address, uchar value);
	
	// ROM BANK SET
	static void WriteROM(void* user, int address, uchar value);
	
	// LOOP
	static uchar Loop(void* user);
};

#endif
